
include

Reset:
	SEI                       ; Disable IRQ
	STZ $4200                 ; Disable IRQ, NMI and joypad reading
	STZ $420C                 ; Disable HDMA
	STZ $420B                 ; Disable DMA
	STZ $2140                 ;\ Clear APU I/O ports (1-4) (End music)
	STZ $2141                 ;|
	STZ $2142                 ;|
	STZ $2143                 ;/
	LDA #$80                  ;\ Activate force blank (lowest brightness)
	STA $2100                 ;/
	CLC                       ;\ Disable 6502 emulation mode
	XCE                       ;/

	STZ $2183				; WRAM Address bank bit
	STZ $2121				; CGRAM Address

	LDA #%10000000	; Increment when $2119 or $213A is accessed, Increment by 2 bytes
	STA $2115		;

	REP #$28				; Disable decimal mode; widen A to 16-bit

	LDA #$0000                ;\ Set the Direct Page $000000-FF
	TCD                       ;/             (mirror of $7E0000-FF)
	LDA #$1FFF                ;\ Set the Stack Pointer
	TCS                       ;/

	STZ $2181				; WRAM Address low byte, high byte
	STZ $2116				; VRAM Address


!channel = $00
; DMA Channel 0, WRAM
	LDA #$8000|%00001000			; d--aauuu   d=direction, A to B; a=auto update, no; u=unit size and transfer format, 1 register write once
	STA $4300|!channel				; DMA Control for Channel 0, DMA Destination Register for Channel 0
	LDA #EmptyData
	STA $4302|!channel				; DMA Source Address for Channel 0 low byte, high byte
	LDX.b #bank(EmptyData)
	STX $4304|!channel				; DMA Source Address for Channel 0 bank byte
	LDA #$2000
	STA $4305|!channel				; DMA Size for Channel 0 low byte, $2000 bytes

; DMA Channel 1, CGRAM
!channel = $10
	LDA #$2200|%00000010			; d--aauuu   d=direction, A to B; a=auto update, increment; u=unit size and transfer format, 1 register write twice
	STA $4300|!channel				; DMA Control for Channel 1, DMA Destination Register for Channel 0
	LDA #Palette
	STA $4302|!channel				; DMA Source Address for Channel 1 low byte, high byte
	LDX.b #bank(Palette)
	STX $4304|!channel				; DMA Source Address for Channel 1 bank byte
	LDA #$0200						; Transfer $200 bytes, the size of CGRAM
	STA $4305|!channel				; DMA Size for Channel 1 low byte

; DMA Channel 2, VRAM
!channel = $20
	LDA #$1800|%00000001			; d--aauuu   d=direction, A to B; a=auto update, increment; u=unit size and transfer format, 2 registers write once
	STA $4300|!channel				; DMA Control for Channel 2, DMA Destination Register for Channel 0
	LDA #Tilesets
	STA $4302|!channel				; DMA Source Address for Channel 2 low byte, high byte
	LDX.b #bank(Tilesets)
	STX $4304|!channel				; DMA Source Address for Channel 2 bank byte
	LDA #Tilesets_end-Tilesets		; Transfer all tileset data
	STA $4305|!channel				; DMA Size for Channel 2 low byte, 0x200 bytes

	LDX #%00000111			; channels 0, 1, 2
	STX $420B				; DMA Enable

!channel = $10
	LDA #$1800|%00001001			; d--aauuu   d=direction, A to B; a=auto update, no; u=unit size and transfer format, 2 registers write once
	STA $4300|!channel				; DMA Control for Channel 1, DMA Destination Register for Channel 1

	REP #$10                  ; widen X and Y to 16-bit

	LDX #$2101
	;regs $2101-$210C
-	STZ $00,X		;set Sprite,Character,Tile sizes to lowest, and set addresses to $0000
	INX #2
	CPX #$210D
	BNE -

	;regs $210D-$2114
-	STZ $00,X		;Set all BG scroll values to $0000
	STZ $00,X
	INX #2
	CPX #$2115
	BNE -

	LDX #$211B
	;regs $211B-$2120
-	STZ $00,X		;clear out the Mode7 matrix values
	STZ $00,X
	INX #2
	CPX #$2121
	BNE -

	;reg $2121 - Color address, doesn't need initilaizing
	;reg $2122 - Color data, is initialized later

	LDX #$2123
	;regs $2123-$2133
-	STZ $00,X		;turn off windows, main screens, sub screens, color addition,
	INX	#2		;fixed color = $00, no super-impose (external synchronization),
	CPX #$2133	;no interlaced mode, normal resolution
	BNE -

	SEP #$30

	;VRAM stuff
	LDA #%00001001	; Mode 1, Layer 3 priority
	STA !BGmode		;
	LDA #%00010111	; Enable Layer 1, 2, 3, obj on Main Screen
	STA !mainscreen
	STA !mainscreenwindow
;	LDA #%00010110	; Enable Layer 1, 3, obj on Main Screen
;	STA $212D		;
;	STA !subscreen

!tilemapstart = Tilemaps-Tilesets
;!tilemapsize = $0800

	LDA.b #!tilemapstart>>9	; 
	STA $2107		; BG1 Tilemap Address and Size, aaaaaayx

	LDA.b #!tilemapstart>>9+4	; 
	STA $2108		; BG2 Tilemap Address and Size, aaaaaayx

	LDA.b #!tilemapstart>>9+8	; 
	STA $2109		; BG3 Tilemap Address and Size, aaaaaayx

	STZ $210B		; BG1 and 2 Chr Address
	LDA #%00000001
	STA $210C		; BG3 and 4 Chr Address


	STZ $211A
	;LDA #%00001000
	STZ $2133

	;regs $2134-$2136  - multiplication result, no initialization needed
	;reg $2137 - software H/V latch, no initialization needed
	;reg $2138 - Sprite data read, no initialization needed
	;regs $2139-$213A  - VRAM data read, no initialization needed
	;reg $213B - Color RAM data read, no initialization needed
	;regs $213C-$213D  - H/V latched data read, no initialization needed

	STZ $213E		;reg $213E - might not be necesary, but selects PPU master/slave mode
	;reg $213F - PPU status flag, no initialization needed

	;reg $2140-$2143 - APU communication regs, no initialization required

	;reg $2180  -  read/write WRAM register, no initialization required
	;reg $2181-$2183  -  WRAM address, no initialization required

	;reg $4016-$4017  - serial JoyPad read registers, no need to initialize

	LDA #$FF
	STA $4201		;reg $4201  - programmable I/O write port, initalize to allow reading at in-port

	;regs $4202-$4203  - multiplication registers, no initialization required
	;regs $4204-$4206  - division registers, no initialization required

	;regs $4207-$4208  - Horizontal-IRQ timer setting, since we disabled this, it is OK to not init
	;regs $4209-$420A  - Vertical-IRQ timer setting, since we disabled this, it is OK to not init

	STZ $420D		;reg $420D  - ROM access time to slow (2.68Mhz)

	LDA $4210		;reg $4210  - NMI status, reading resets

	;reg $4211  - IRQ status, no need to initialize
	;reg $4212  - H/V blank and JoyRead status, no need to initialize
	;reg $4213  - programmable I/O inport, no need to initialize

	;reg $4214-$4215  - divide results, no need to initialize
	;reg $4216-$4217  - multiplication or remainder results, no need to initialize

	;regs $4218-$421f  - JoyPad read registers, no need to initialize

	;regs $4300-$437F
	;no need to intialize because DMA was disabled above

	REP #$20

	LDA #$FFFF
	STA !BG1ypos
	STA !BG2ypos
	STA !BG3ypos

	SEP #$20				; get 8-bit registers

	;SRAM
	LDA #$76
	CMP $F00000
	BEQ ++

	;first time
	STA $F00000

	LDA #$02
	STA !language
	STA $F00001

	BRA +

	;load settings
++	LDA $F00001
	STA !language
	LDA $F00002
	STA !turnorder
	LDA $F00003
	STA !vsCPU

+	CLI						; enable interrupts
	LDA #%10000001			;
	STA $4200				; enable NMI and joypad reading

	STZ !brightness

;	PHK
;	PLB

	;debug
;	LDA #$08
;	STA !gamemode
	
	JMP GameLoop

EmptyData:
db $00
